module Fedi.Routes.Notes where

import Data.Aeson qualified as A
import Fedi.Helpers
import Fedi.Routes.Helpers
import Fedi.Types
import Fedi.UserDetails
import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain

-- * Create

matchCreateNote :: UserDetails -> Twain.PathPattern
matchCreateNote details = fromString ("/" <> details.username <> "/notes/:note_id/create")

handleCreateNote :: UserDetails -> [Create] -> Twain.ResponderM a
handleCreateNote details items = do
  noteId <- Twain.param "note_id"
  let
    createUrl =
      "https://"
        <> details.domain
        <> "/"
        <> details.username
        <> "/notes/"
        <> noteId
        <> "/create"
  let
    content =
      find (\create -> create.id == Just (ObjectId createUrl)) items
  Twain.send $ jsonLD (A.encode content)

-- * Note

matchNote :: UserDetails -> Twain.PathPattern
matchNote details = fromString ("/" <> details.username <> "/notes/:note_id")

handleNote :: UserDetails -> [Note] -> Twain.ResponderM a
handleNote details items = do
  noteId <- Twain.param "note_id"
  let
    noteUrl =
      "https://"
        <> details.domain
        <> "/"
        <> details.username
        <> "/notes/"
        <> noteId
  let
    content =
      find (\note -> note.id == Just (ObjectId noteUrl)) items
  Twain.send $ jsonLD (A.encode content)
