{-# LANGUAGE RecordWildCards #-}

module Fedi.Signature.Sign (
  module Fedi.Signature.Types,
  module Fedi.Signature.Sign,
) where

import Data.ByteString qualified as BS
import Data.Time qualified as Time
import Fedi.Crypto
import Fedi.Helpers
import Fedi.Signature.Types
import Fedi.UserDetails
import Prelude hiding (error)

-- * Sign

signSignature
  :: UserDetails -> String -> String -> ByteString -> IO HttpSignature
signSignature details host requestTarget body = do
  date <-
    Time.getCurrentTime
      <&> Time.formatTime Time.defaultTimeLocale "%a, %d %b %Y %H:%M:%S GMT"
  let
    digest = "SHA-256=" <> encodeBase64 (makeDigest body)
    keyId = actorUrl details <> "#main-key"
    headers = ["(request-target)", "host", "date", "digest"]
    components = []
    signatureString = makeSignatureString host requestTarget date digest
  signed <- sign details.privatePem signatureString
  let
    signature = bsSignedMessage signed
    signatureHeader = SignatureHeader {..}
  pure HttpSignature {..}

makeSignatureString
  :: String -> String -> String -> ByteString -> ByteString
makeSignatureString host requestTarget date digest =
  BS.intercalate
    "\n"
    [ "(request-target): " <> fromString requestTarget
    , "host: " <> fromString host
    , "date: " <> fromString date
    , "digest: " <> digest
    ]
