{-# LANGUAGE RecordWildCards #-}

module Fedi.Signature.Types where

import Data.ByteString qualified as BS
import Data.Text qualified as T
import Data.Text.Encoding qualified as T
import Fedi.Helpers
import Fedi.UserDetails
import Prelude hiding (error)

data HttpSignature
  = HttpSignature
  { signatureHeader :: SignatureHeader
  , date :: String
  , host :: String
  , digest :: ByteString
  }
  deriving (Show)

toSignature :: SignatureHeader -> ByteString
toSignature sig =
  BS.intercalate
    ","
    [ "keyId=\"" <> fromString sig.keyId <> "\""
    , "headers=\"" <> BS.intercalate " " (map T.encodeUtf8 sig.headers) <> "\""
    , "signature=\"" <> sig.signature <> "\""
    , "algorithm=\"" <> "rsa-sha256" <> "\""
    ]

data SignatureHeader
  = SignatureHeader
  { keyId :: Url
  -- ^ Where to get the public key for this actor
  , headers :: [T.Text]
  -- ^ Which headers have been sent
  , signature :: ByteString
  -- ^ Contains the signature
  , components :: [(Component, String)]
  }
  deriving (Show)

data Component
  = KeyId
  | Headers
  | Signature
  | Algorithm
  | Other String
  deriving (Eq, Show)
